#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var yargs = require("yargs");
var graphql_1 = require("graphql");
var fs = require("fs");
var chalk = require("chalk");
var mkdirp = require("mkdirp");
var source_helper_1 = require("./source-helper");
var path_1 = require("path");
var ts_generator_1 = require("./generators/ts-generator");
var reason_generator_1 = require("./generators/reason-generator");
var ts_scaffolder_1 = require("./generators/ts-scaffolder");
var reason_scaffolder_1 = require("./generators/reason-scaffolder");
var graphql_import_1 = require("graphql-import");
function getGenerator(generator) {
    if (generator === "interfaces-reason") {
        return { generate: reason_generator_1.generate, format: reason_generator_1.format };
    }
    if (generator === "scaffold-reason") {
        return { generate: reason_scaffolder_1.generate, format: reason_generator_1.format };
    }
    if (generator === "interfaces-typescript") {
        return { generate: ts_generator_1.generate, format: ts_generator_1.format };
    }
    if (generator === "scaffold-typescript") {
        return { generate: ts_scaffolder_1.generate, format: ts_generator_1.format };
    }
    return { generate: ts_generator_1.generate, format: ts_generator_1.format };
}
function generateCode(_a) {
    var _b = _a.schema, schema = _b === void 0 ? undefined : _b, _c = _a.prettify, prettify = _c === void 0 ? true : _c, _d = _a.generator, generator = _d === void 0 ? "interfaces-typescript" : _d;
    if (!schema) {
        console.error(chalk.default.red("Please provide a parsed GraphQL schema"));
    }
    var generateArgs = {
        types: source_helper_1.extractGraphQLTypes(schema),
        enums: source_helper_1.extractGraphQLEnums(schema),
        unions: source_helper_1.extractGraphQLUnions(schema)
    };
    var generatorFn = getGenerator(generator);
    var code = generatorFn.generate(generateArgs);
    if (typeof code === "string") {
        if (prettify) {
            return generatorFn.format(code);
        }
        else {
            return code;
        }
    }
    else {
        return code.map(function (f) {
            return {
                path: f.path,
                code: prettify ? generatorFn.format(f.code) : f.code
            };
        });
    }
}
exports.generateCode = generateCode;
function run() {
    var defaults = {
        output: "./generated/resolvers",
        generator: "typescript",
        interfaces: "./generated/"
    };
    var argv = yargs
        .usage("Usage: <command> $0 -s [schema-path] -o [output-path] -g [generator] -i [interfaces]")
        .alias("s", "schema-path")
        .describe("s", "GraphQL schema file path")
        .alias("o", "output")
        .describe("o", "Output file/folder path [default: " + defaults.output + "[.ts]]")
        .alias("g", "generator")
        .describe("g", "Generator to use [default: " + defaults.generator + ", options: reason]")
        .describe("i", "Path to the interfaces folder used for scaffolding")
        .alias("i", "interfaces") // TODO: Make this option only be used with scaffold command
        .demandOption(["s"])
        .demandCommand(1)
        .strict().argv;
    var command = argv._[0];
    if (["scaffold", "interfaces"].indexOf(command.toLowerCase()) <= -1) {
        console.error("Unknown command provided, please provide either scaffold or interfaces as the command");
        process.exit(1);
    }
    var args = {
        command: command,
        schemaPath: path_1.resolve(argv.schemaPath),
        output: argv.output ||
            "" + defaults.output + (command === "interfaces" ? ".ts" : ""),
        generator: argv.generator || defaults.generator,
        interfaces: argv.interfaces || defaults.interfaces
    };
    // TODO: Do a check on interfaces if provided
    if (!fs.existsSync(args.schemaPath)) {
        console.error("The schema file " + args.schemaPath + " does not exist");
        process.exit(1);
    }
    var schema = undefined;
    try {
        schema = graphql_import_1.importSchema(args.schemaPath);
    }
    catch (e) {
        console.error(chalk.default.red("Error occurred while reading schema: " + e));
        process.exit(1);
    }
    var parsedSchema = undefined;
    try {
        parsedSchema = graphql_1.parse(schema);
    }
    catch (e) {
        console.error(chalk.default.red("Failed to parse schema: " + e));
        process.exit(1);
    }
    var code = generateCode({
        schema: parsedSchema,
        generator: args.command + "-" + args.generator
    });
    if (typeof code === "string") {
        // Create generation target folder, if it does not exist
        // TODO: Error handling around this
        mkdirp.sync(path_1.dirname(args.output));
        try {
            fs.writeFileSync(args.output, code, { encoding: "utf-8" });
        }
        catch (e) {
            console.error(chalk.default.red("Failed to write the file at " + args.output + ", error: " + e));
            process.exit(1);
        }
        console.log(chalk.default.green("Code generated at " + args.output));
        process.exit(0);
    }
    else {
        // Create generation target folder, if it does not exist
        // TODO: Error handling around this
        mkdirp.sync(path_1.dirname(args.output));
        code.forEach(function (f) {
            var writePath = path_1.join(args.output, f.path);
            try {
                fs.writeFileSync(writePath, f.code.replace("[TEMPLATE-INTERFACES-PATH]", args.interfaces), {
                    encoding: "utf-8"
                });
            }
            catch (e) {
                console.error(chalk.default.red("Failed to write the file at " + args.output + ", error: " + e));
                process.exit(1);
            }
            console.log(chalk.default.green("Code generated at " + writePath));
        });
        process.exit(0);
    }
}
// Only call run when running from CLI, not when included for tests
if (require.main === module) {
    run();
}
//# sourceMappingURL=index.js.map